// Note: adapted from a basic
// vector2 class from an assignment
#include "C_Vector.h"
#include <cmath>
#include <assert.h>

// defines PI
#define PI (4.0 * atan(1.0))

// Constructors
C_Vector::C_Vector(void) : x(0),y(0) {}
C_Vector::C_Vector(float xin, float yin) : x(xin), y(yin) {}
C_Vector::C_Vector(const C_Vector &other) : x(other.x), y(other.y) {}

// destructor
C_Vector::~C_Vector(void){}

// Operators
const C_Vector & C_Vector::operator -=(const C_Vector &rhs)
{
	x -= rhs.x;
	y -= rhs.y;
	return *this;
}

const C_Vector & C_Vector::operator +=(const C_Vector &rhs)
{
	x += rhs.x;
	y += rhs.y;
	return *this;
}

const C_Vector & C_Vector::operator /=(const C_Vector &rhs)
{
	x /= rhs.x;
	y /= rhs.y;
	return *this;
}

const C_Vector & C_Vector::operator /=(const float &f)
{
	x /= f;
	y /= f;
	return *this;
}

const C_Vector & C_Vector::operator *=(const C_Vector &rhs)
{
	x *= rhs.x;
	y *= rhs.y;
	return *this;
}

const C_Vector & C_Vector::operator *=(const float &f)
{
	x *= f;
	y *= f;
	return *this;
}

const C_Vector & C_Vector::operator =(const C_Vector &rhs)
{
	if(&rhs != this)
	{
		x = rhs.x;
		y = rhs.y;
	}

	return *this;
}

const C_Vector C_Vector::operator -(const C_Vector &rhs) const
{
	C_Vector ret(*this);
	ret -= rhs;
	return ret;
}

const C_Vector C_Vector::operator +(const C_Vector &rhs) const
{
	C_Vector ret(*this);
	ret += rhs;
	return ret;
}

const C_Vector C_Vector::operator /(const C_Vector &rhs) const
{
	C_Vector ret(*this);
	ret /= rhs;
	return ret;
}

const C_Vector C_Vector::operator /(const float &fl) const
{
	C_Vector ret(*this);
	ret.x /= fl;
	ret.y /= fl;
	return ret;
}

const C_Vector C_Vector::operator *(const C_Vector &rhs) const
{
	C_Vector ret(*this);
	ret *= rhs;
	return ret;
}

const C_Vector C_Vector::operator *(const float &fl) const
{
	C_Vector ret(*this);
	ret.x *= fl;
	ret.y *= fl;
	return ret;
}

bool C_Vector::operator == (const C_Vector& c) const 
{
	if (c.x != x || c.y != y)
		return false;

	return true;
}

bool C_Vector::operator != (const C_Vector& c) const
{
	if (c.x == x || c.y == y)
		return false;

	return true;
}

// Functions

// Normalise this vector
void C_Vector::Normalise() 
{
	float len = Length();
	if(len == 0) 
		return;

	x /= len;
	y /= len;
}

// Calculates cosine angle between two vectors
float C_Vector::Dot(const C_Vector& other) const
{
	return x * other.x + y * other.y;
}

// Calculates the 2D cross product of 2 vectors resulting in a float which is 2 times the the area of the triangle
float C_Vector::Cross(const C_Vector& other) const
{
    return x * other.y - y * other.x;
}

// Rotates the vector around a specified angle
void C_Vector::Rotate(const float &angle)
{
	// error handling statements
	// asserting certain statements for this function
	assert(angle >= 0 && angle <= 360);

	// performs the rotation
	float new_x = x * cos(angle) - y * sin(angle);
	float new_y = y * sin(angle) + x * cos(angle);

	// alters the parameters of the vector
	x = new_x;
	y = new_y;
}

// Returns the angle between this vector and another specified vector
float C_Vector::AngleBetween(const C_Vector &other)
{
	// optimises when the vectors are the same
	if(other != *this)
	{
		float dotProduct = Dot(other);

		// returns a right angle when the dot product is zero
		if(dotProduct != 0)
		{
			// calculates the angle between the two vectors
			float radians = acos(dotProduct / (Length() * other.Length()));

			// converts the answer to degrees
			return ((float) (radians * (180 / PI)));
		}
		else
		{
			return 90;
		}
	}
	else
	{
		return 0;
	}
}


