// Note: adapted from a basic
// vector2 class from an assignment
#pragma once


#include <cmath>

class C_Vector
{
public:
	float x,y;

	// Constructors
	C_Vector(void);
	C_Vector(float xin, float yin);
	C_Vector(const C_Vector &other);

	// Operators
	const C_Vector & operator -=(const C_Vector &rhs);
	const C_Vector & operator +=(const C_Vector &rhs);
	const C_Vector & operator /=(const C_Vector &rhs);
	const C_Vector & operator /=(const float &f);
	const C_Vector & operator *=(const C_Vector &rhs);
	const C_Vector & operator *=(const float &f);
	const C_Vector & operator =(const C_Vector &rhs);
	const C_Vector operator -(const C_Vector &rhs) const;
	const C_Vector operator +(const C_Vector &rhs) const;
	const C_Vector operator /(const C_Vector &rhs) const;
	const C_Vector operator /(const float &fl) const;
	const C_Vector operator *(const C_Vector &rhs) const;
	const C_Vector operator *(const float &fl) const;
	bool operator == (const C_Vector& c) const;
	bool operator != (const C_Vector& c) const;

	// Functions

	// Vector length
	float Length() const {return sqrt(x * x + y * y);} const

	// Squared length
	float SquaredLength() const;

	// Normalise this vector
	void Normalise();

	// Calculates cosine angle between two vectors
	float Dot(const C_Vector& other) const;

	// Calculates the 2D cross product of 2 vectors resulting in a float which is 2 times the the area of the triangle
	float Cross( const C_Vector& other) const;

	// rotates this vector around a specified angle
	void Rotate(const float &angle);

	// calculates the angle between this vector and another vector
	float AngleBetween(const C_Vector &other);

	~C_Vector(void);
};

