// created by Josh Heyde
// last updated 21/03/09

#pragma once

// forward declarations
class C_Vector;
class C_Rectangle;

#include <vector>

class C_WorldEntity
{
public:
	virtual ~C_WorldEntity();

	// enum for type of flocking entity
	const enum eType{ePrey, ePredator};

	// accessor and mutator functions
	const bool getActive() const {return active;}
	virtual void setActive(const bool &newActive);
	const int getGraphicID() const {return graphicID;}
	void setGraphicID(const int &newGraphicID);
	const int getSpeed() const {return speed;}
	void setSpeed(const int &newSpeed);
	const int getMaxVeloctiy() const {return maxVelocity;}
	void setMaxVelocity(const int &newMaxVelocity);
	const int getRepulsionRange() const {return repulsionRange;}
	void setRepulsionRange(const int &newRepulsionRange);
	const int getVisionRange() const {return visionRange;}
	void setVisionRange(const int &newVisionRange);
	const int getTurnSpeed() const {return turnSpeed;}
	void setTurnSpeed(const int &newTurnSpeed);
	const C_Vector* getCurrentPosition() const {return currentPosition;}
	void setCurrentPosition(const int &newX, const int &newY);
	void setCurrentPosition(const C_Vector *newVector);
	const C_Vector* getNextPosition() const {return nextPosition;}
	void setNextPosition(const int &newX, const int &newY);
	void setNextPosition(const C_Vector *newVector);
	const C_Vector* getDirection() const {return direction;}
	void setDirection(const int &newX, const int &newY);
	void setDirection(const C_Vector* newDirection);
	const C_Vector* getDesiredDirection() const {return desiredDirection;}
	void setDesiredDirection(const int &newX, const int &newY);
	void setDesiredDirection(const C_Vector* newDesiredDirection);
	const C_Rectangle* getBoundaries() const {return boundaries;}
	void setBoundaries(const int &newLeft, const int &newRight, const int &newTop, const int &newBottom);
	void setBoundaries(const C_Rectangle *newRect);
	const eType getType() const {return type;}
	void setType(const eType &newType);

	// other member functions
	bool checkCollision(C_WorldEntity* other);
	virtual void collisionResult() =0;
	void controlVelocity();
	virtual void determineDirection(std::vector <C_WorldEntity*> v_entityVector) =0;
	bool inRange(const C_WorldEntity *other, int range) const;
	virtual bool isEnemyOf(C_WorldEntity* other) const =0;
	virtual void render(const float &currentTime) =0;
	virtual void update();

protected:
	// the constructor is protected to prevent any
	// instantiation of this class, but still allows
	// child classes to use it
	C_WorldEntity(const int &ID, const int &cF, const float &pX, const float &pY, const int &s, const int &mV, 
		const int &rR, const int &vR, const eType& newType);

	// data members
	bool active;
	int graphicID;
	int currentFrame;
	int speed;
	int maxVelocity;
	int repulsionRange;
	int visionRange;
	int turnSpeed;
	C_Vector* currentPosition;
	C_Vector* nextPosition;
	C_Vector* direction;
	C_Vector* desiredDirection;
	C_Rectangle* boundaries;
	eType type;
};